#ifndef __VMSTERM_LOADED
#define __VMSTERM_LOADED	1
#ifdef VMS


/*
 *	VMSTERM.H
 *
 *	Copyright (c) 1991 by Michael S Zraly.
 *
 *	Some of the details in this code were derived from code
 *	for the games larn, copyright 1986 by Noah Morgan (as
 *	adapted by Kevin Routley), and nethack, copyright 1985
 *	by the Stichting Mathematisch centrum, Amsterdam.
 *
 *	Permission is granted to distribute and modify this source
 *	freely, provided that no charge is made for doing so and
 *	that this comment remains untouched.
 *
 *	Last Modified:	21 August 1991
 */


#include	<iodef.h>
#include	<ssdef.h>
#include	<ttdef.h>
#include	<tt2def.h>


/*
 *	Return codes:
 */
#define	TT_SUCCESS	1
#define	TT_FAILURE	0
#define	TT_ERROR	(-1)
#define	TT_TIMEOUT	(-2)


/*
 *	These speed codes are for compatability with BSD UNIX
 */
#define	B0	(-1)			/* no VMS equivalent */
#define	B50	TT$C_BAUD_50
#define	B75	TT$C_BAUD_75
#define	B110	TT$C_BAUD_110
#define	B134	TT$C_BAUD_134
#define	B150	TT$C_BAUD_150
#define	B200	B150			/* no VMS equivalent */
#define	B300	TT$C_BAUD_300
#define	B600	TT$C_BAUD_600
#define	B1200	TT$C_BAUD_1200
#define	B1800	TT$C_BAUD_1800
#define	B2000	TT$C_BAUD_2000		/* no BSD equivalent */
#define	B2400	TT$C_BAUD_2400
#define	B3600	TT$C_BAUD_3600		/* no BSD equivalent */
#define	B4800	TT$C_BAUD_4800
#define	B7200	TT$C_BAUD_7200		/* no BSD equivalent */
#define	B9600	TT$C_BAUD_9600
#define	B19200	TT$C_BAUD_19200		/* no BSD equivalent */
#define	B38400	TT$C_BAUD_38400		/* no BSD equivalent */
#define	EXTA	B50			/* no VMS equivalent */
#define	EXTB	B50			/* no VMS equivalent */

#define	BAUD_MIN	B50
#define	BAUD_MAX	B38400


/*
 *	This is the i/o status block structure returned from
 *	an IO$_READxBLK call to SYS$QIO() or SYS$QIOW().
 */
typedef struct {
   unsigned status	: 16;
   unsigned term_offset	: 16;
   unsigned terminator	: 16;
   unsigned term_size	: 16;
} IOSB_R;


/*
 *	This is the i/o status block structure returned from
 *	an IO$_SENSEMODE or IO$_SETMODE call to SYS$QIO() or
 *	SYS$QIOW().
 */
typedef struct {
   unsigned status	: 16;
   unsigned ospeed	:  8;
   unsigned ispeed	:  8;
   unsigned cr_fill	:  8;
   unsigned lf_fill	:  8;
   unsigned parity	:  8;
   unsigned unused	:  8;
} IOSB_S;


/*
 *	This is the structure describing terminal characteristics
 *	passed as the P1 argument in an IO$_SENSEMODE or IO$_SET-
 *	MODE call to SYS$QIO() or SYS$QIOW().
 */
typedef struct {
   unsigned class	:  8;
   unsigned type	:  8;
   unsigned cols	: 16;
   unsigned ttchar	: 24;	/* see <ttdef.h> */
   unsigned rows	:  8;
   unsigned tt2char	: 32;	/* see <tt2def.h> */
} TTBUF;


/*
 *	Macro functions to get and
 *	set terminal characteristics
 */
#define	tt_getmode(iosb_s,ttbuf)				\
	( ((SYS$QIOW(0, tt_chan(), IO$_SENSEMODE, &(iosb_s),	\
	(void(*)())0, 0, &(ttbuf), sizeof(ttbuf), 0, 0, 0, 0)	\
	& SS$_NORMAL) == 0) ? TT_FAILURE : TT_SUCCESS )

#define	tt_setmode(iosb_s,ttbuf)				\
	( ((SYS$QIOW(0, tt_chan(), IO$_SETMODE, &(iosb_s),	\
	(void(*)())0, 0, &(ttbuf), sizeof(ttbuf), 0, 0, 0, 0)	\
	& SS$_NORMAL) == 0) ? TT_FAILURE : TT_SUCCESS )


/*
 *	Prototypes:
 */
int	tt_chan();
int	tt_init();
int	tt_getc();
void	tt_flush();
void	tt_fini();
int	tt_speed();
int	tt_cols();
int	tt_rows();
void	tt_shell();


#endif					/* VMS */
#endif					/* __VMSTERM_LOADED */
